#pragma once
#pragma warning(disable:4786)


#include <iostream>
#include <cassert>
#include <map>
#include <string>
#include <vector>


enum Type
{
	AT_INT,
	AT_FLOAT,
	AT_DOUBLE,
	AT_STRING
};



class CAttribute
{
	/// the node manager has the right to manipulate the internal attribute data
	friend class CNodeManager;

public:

	
	CAttribute(int *p,bool bd = false) ;
	CAttribute(float *p,bool bd = false) ;
	CAttribute(double *p,bool bd = false) ;
	CAttribute(std::string *p,bool bd = false) ;

	/// returns the text name of the attribute
	const std::string GetName() const ;

	/// returns true if we should delete the data when dtor is called
	bool IsDynamicAttr() const;

	/// returns the value as an int
	int AsInt() ;

	/// returns the value as a float
	float AsFloat();

	/// returns the value as a double
	double AsDouble() ;

	/// returns the value as a string
	std::string AsString() ;
	
	/// sets the value
	void Set(int v) ;

	/// sets the value
	void Set(float v) ;

	/// sets the value
	void Set(double v) ;

	/// sets the value
	void Set(std::string v) ;

	const Type GetType() const {
		return m_Type;
	}


private:


	/// the name of the attribute
	std::string m_Name;

	/// the type of the attribute
	Type m_Type;

	/// was the attribute created by us?
	bool m_bDynamic;

	/// pointers to the variables. Because attributes can only
	/// have one type, I'm using a union
	union 
	{
		int *i;
		float *f;
		double *d;
		std::string *s;
	};
};
