#pragma once

#include "Attribute.h"

/// use this helper macro inside your constructor
#define DECLARE_ATTR(X) AddAttr( &##X , std::string( #X ) );

/// \brief	The base class of all nodes
/// 
class CNode
{
	/// the node manager has the right to manipulate the internal attribute data
	friend class CNodeManager;

	/// the attribute map type
	typedef std::map< std::string, CAttribute > AttrMap;

	/// attribute iterator
	typedef AttrMap::iterator AttrIt;

	/// the attributes for this node
	AttrMap m_Attributes;

protected:

	/// these functions are called by inherited classes via the DECLARE_ATTR macro
	void AddAttr(int* p,std::string name) {
		m_Attributes.insert( std::make_pair(name, CAttribute(p) ) );
	}
	void AddAttr(float* p,std::string name)	{
		m_Attributes.insert( std::make_pair(name, CAttribute(p) ) );
	}
	void AddAttr(double* p,std::string name) {
		m_Attributes.insert( std::make_pair(name, CAttribute(p) ) );
	}
	void AddAttr(std::string* p,std::string name) {
		m_Attributes.insert( std::make_pair(name, CAttribute(p) ) );
	}

	CNode(void);
	~CNode(void);

public:


	/// call to add a new dynamic scripted attribute to the node.
	/// \param	"int", "float", "double" or "string"
	/// \param	the attribute name
	/// 
	bool AddAttr(std::string ,std::string );

	/// call to delete a dynamic attribute. This call fails if attempting 
	/// to delete a static attribute
	bool DelAttr(std::string );

	/// two get funcs for lua
	bool GetAttr(std::string , std::string &);
	bool GetAttr(std::string , double &);
	Type GetAttrType(std::string);

	/// two set funcs for lua
	bool SetAttr(std::string , std::string );
	bool SetAttr(std::string , double );

	/// shitty func to print the attributes out
	void ListAttr();
};
