#include "nodemanager.h"

CNodeManager::CNodeManager(void)
{
}

CNodeManager::~CNodeManager(void)
{
}

CNode* CNodeManager::CreateNode(std::string type,std::string name)
{
	/// first see if a node of the same name exists...
	if (find(name) != end()) 
	{
		std::cerr << "[ !ERROR! ] - node " 
			<< name 
			<< " already exists" 
			<< std::endl;
		return NULL;
	}

	/// next see if we have a creator function for the requested type
	CreatorIt it = CreatorFunctionMap.find(type);

	if (it != CreatorFunctionMap.end()) {

		// call the creator function

		CNode* pnode = it->second();

		insert( std::make_pair(name,pnode) );

		return pnode;			
	}

	return NULL;
}


bool CNodeManager::RegisterNodeType(std::string name,NodeCreateFunc fn)
{
	/// next see if we have a creator function for the requested type
	CreatorIt it = CreatorFunctionMap.find(name);

	if (it != CreatorFunctionMap.end()) {
		return false;
	}

	CreatorFunctionMap.insert( std::make_pair(name,fn) );

	return true;
}	

bool CNodeManager::DeleteNode(std::string name)
{
	/// first see if a node of the same name exists...
	NodeIt it = find(name);

	if (it != end() ) {

		delete it->second;
		erase(it);
		return true;
	}
	return false;
}

CNode* CNodeManager::GetNode(std::string name) 
{

	/// first see if a node of the same name exists...
	NodeIt it = find(name);

	if (it != end() ) 
	{
		return it->second;
	}
	return NULL;
}

void CNodeManager::ListNodes() 
{
	NodeIt it = begin();
	for( ; it != end() ; ++it ) {
		std::cout << it->first << std::endl; 
	}
}