#pragma once

#include "Node.h"

/// create a function pointer type. The one thing that C++ lacks is a 
/// "virtual constructor". In order to be able to create any registered 
/// node types, you must "register" a node type with a string name and
/// a function that creates a new node 
/// 
typedef CNode* (*NodeCreateFunc)();

/// The nodes in the scene will be stored in a map with their name as 
/// the key
/// 
typedef std::map< std::string, CNode* > NodeMap;

/// An iterator to traverse the nodes
/// 
typedef NodeMap::iterator NodeIt;


/// This class is used to create new nodes 
class CNodeManager : public NodeMap
{
	/// the map type for the creator function
	typedef std::map< std::string , NodeCreateFunc > CreatorMap;
	
	/// the creator function iterator type
	typedef CreatorMap::iterator CreatorIt;

	/// an internal map holding the creator functions
	CreatorMap CreatorFunctionMap;

public:

	/// ctor
	CNodeManager(void);

	/// dtor
	~CNodeManager(void);

	/// this function registers a new node type with the node manager
	bool RegisterNodeType(std::string,NodeCreateFunc);

	/// this function creates a node of the specified name and type
	CNode* CreateNode(std::string,std::string);

	/// this function deletes the specified node
	bool DeleteNode(std::string);

	/// this function returns a pointer to the reqested node
	CNode* GetNode(std::string);

	/// this function lists the attributes to std::cout
	void ListNodes();
};
