

#include "NodeManager.h"


/// the node manager
CNodeManager g_Nodes;

/// the first of two node types
class CPlayer : public CNode
{
public:

	int x,y,z;
	float red,green,blue;
	std::string name;

	CPlayer() : CNode()
	{
		x=y=z=0;
		red=green=blue=0;
		name = "player1";

		/// call this for each attribute in the constructor
		DECLARE_ATTR(x);
		DECLARE_ATTR(y);
		DECLARE_ATTR(z);
		DECLARE_ATTR(red);
		DECLARE_ATTR(green);
		DECLARE_ATTR(blue);
		DECLARE_ATTR(name);
	}
};

/// the creator function for the player class
CNode* CreatePlayer() {
	return new CPlayer;
}

/// the secon node type
class CChimp : public CNode
{
public:

	double bananas;
	double food;
	float beer;
	std::string gardenTwigs;

	CChimp() : CNode()
	{
		bananas=food=beer=0;
		gardenTwigs = "twiggy!";

		/// call this for each attribute in the constructor
		DECLARE_ATTR(bananas);
		DECLARE_ATTR(food);
		DECLARE_ATTR(beer);
		DECLARE_ATTR(gardenTwigs);
	}
};

/// the creator function for the chimp class
CNode* CreateChimp() {
	return new CChimp;
}


void InitNodeManager()
{
	g_Nodes.RegisterNodeType("CPlayer",CreatePlayer);
	g_Nodes.RegisterNodeType("CChimp",CreateChimp);
}











extern "C" {
	#include <lua.h>
	#include <lualib.h>
	#include <lauxlib.h>
}

#pragma comment(lib,"lua.lib")
#pragma comment(lib,"lualib.lib")

lua_State* LUA = NULL;

void ConvertFromString(std::string s,std::string &node,std::string &attr)
{
	std::string val;
	
	node.resize(0);
	attr.resize(0);

	std::string::iterator it = s.begin();

	//	read until the first '.' to indicate the 
	for( ; *it != '.'; ++it )
	{
		assert(it != s.end());
		if (*it != '\t' && *it != ' ' && *it != '\n' ) {
			node += *it;
		}
	}
	// skip '.'
	++it;

	// run through the sring and pull apart the node, attribute and index values
	for( ; it != s.end(); ++it )
	{
		if (*it != '\t' && *it != ' ' && *it != '\n' ) {
			attr += *it;
		}
	}
}


int LCreate(lua_State* L)
{
	int n = lua_gettop(L);

	if(n==2 && lua_isstring(L,1) && lua_isstring(L,2))
	{
		g_Nodes.CreateNode( std::string( lua_tostring(L,1) ) ,std::string( lua_tostring(L,2) ) );
	}
	return 0;
}

int LDelete(lua_State* L)
{
	int n = lua_gettop(L);

	if(n==1 && lua_isstring(L,1) )
	{
		g_Nodes.DeleteNode( std::string( lua_tostring(L,1) ) );
	}
	return 0;
}


int LGetAttr(lua_State* L)
{
	int n = lua_gettop(L);

	if(n==1 && lua_isstring(L,1) )
	{
		std::string s = std::string( lua_tostring(L,1) );
		std::string node,attr;

		ConvertFromString(s,node,attr);

		CNode* pn = g_Nodes.GetNode(node);
		if (pn) {

			if( pn->GetAttrType(attr) == AT_STRING )
			{
				std::string value;
				pn->GetAttr(attr,value);
				lua_pushstring(L,value.c_str());
			}
			else
			if( pn->GetAttrType(attr) <= AT_DOUBLE )
			{
				double value;
				pn->GetAttr(attr,value);
				lua_pushnumber(L,value);
			}
			return 1;
		}		
	}
	return 0;
}


int LSetAttr(lua_State* L)
{
	int n = lua_gettop(L);

	if(n==2 && lua_isstring(L,1) )
	{
		std::string s = std::string( lua_tostring(L,1) );
		std::string node,attr;

		ConvertFromString(s,node,attr);

		CNode* pn = g_Nodes.GetNode(node);
		if (pn) {

			if (lua_isnumber(L,2)) {
				pn->SetAttr(attr,lua_tonumber(L,2));
			}
			else
			if (lua_isstring(L,2)) {
				pn->SetAttr(attr,lua_tostring(L,2));
			}
		}		
	}
	return 0;
}

int LListAttr(lua_State* L)
{
	int n = lua_gettop(L);

	if(n==1 && lua_isstring(L,1) )
	{
		CNode* pn = g_Nodes.GetNode(std::string(lua_tostring(L,1)) );
		if(pn)
		{
			pn->ListAttr();
		}
	}
	return 0;	
}

int LListNodes(lua_State* L)
{
	g_Nodes.ListNodes();
	return 0;	
}

/// args :  node , attr, type
/// 
int LAddAttr(lua_State* L)
{
	int n = lua_gettop(L);

	if(n==3 && lua_isstring(L,1) )
	{
		CNode* pn = g_Nodes.GetNode(std::string(lua_tostring(L,1)) );
		if(pn)
		{
			pn->AddAttr( std::string(lua_tostring(L,3)),std::string(lua_tostring(L,2)) );
		}
	}
	return 0;	
}

int LDelAttr(lua_State* L)
{
	int n = lua_gettop(L);

	if(n==1 && lua_isstring(L,1) )
	{
		std::string s = std::string( lua_tostring(L,1) );
		std::string node,attr;

		ConvertFromString(s,node,attr);

		CNode* pn = g_Nodes.GetNode(node);
		if(pn)
		{
			pn->DelAttr(attr);
		}
	}
	return 0;	
}











void main()
{
	LUA = lua_open();

	lua_baselibopen(LUA);	
	lua_tablibopen(LUA);	
	lua_iolibopen(LUA);	
	lua_strlibopen(LUA);	
	lua_mathlibopen(LUA);	

	InitNodeManager();

	lua_register(LUA,"getAttr",LGetAttr);
	lua_register(LUA,"setAttr",LSetAttr);
	lua_register(LUA,"listAttr",LListAttr);
	lua_register(LUA,"createNode",LCreate);
	lua_register(LUA,"deleteNode",LDelete);
	lua_register(LUA,"addAttr",LAddAttr);
	lua_register(LUA,"delAttr",LDelAttr);
	lua_register(LUA,"ls",LListNodes);

	lua_dofile(LUA, "test.lua" );
	
	lua_close(LUA);
}

